@@ -10,6 +10,8 @@ import warnings
 
 from .GanjaScene import GanjaScene
 
+from .color import Color
+
 CEFAVAILABLE = False
 try:
     from .cefwindow import *
@@ -45,8 +47,9 @@ def read_ganja():
     return output
 
 
-def generate_notebook_js(scene, sig=None, grid=True, scale=1.0, gl=True):
-    script_json = _to_scene_string(scene)
+def generate_notebook_js(scene, sig=None, *,
+                         default_color=Color.DEFAULT, default_static=False, **kwargs):
+    script_json = _to_scene_string(scene, default_color=default_color, default_static=default_static)
     if sig is not None:
         p = (sig > 0).sum().item()  # convert to json-compatible scalar
         q = (sig < 0).sum().item()  # convert to json-compatible scalar
@@ -64,13 +67,25 @@ def generate_notebook_js(scene, sig=None, grid=True, scale=1.0, gl=True):
         conformal = False
 
     if (p, q, r) in [(4, 1, 0), (3, 1, 0), (3, 0, 0), (3, 0, 1)]:
-        if p - q == 2:
-            gl = False  # 2d
         opts = dict(
-            p=p, q=q, r=r, gl=gl, conformal=conformal, grid=grid, scale=scale,
-            mv_length=mv_length
+            p=p, q=q, r=r, mv_length=mv_length,
+            graph=dict(
+                conformal=conformal,
+                gl=True,
+                useUnnaturalLineDisplayForPointPairs=True,
+            )
         )
-        js = read_ganja()
+        if p - q == 2:
+            kwargs['gl'] = False  # 2d
+        opts["graph"].update(kwargs)
+        js = """
+        // We load ganja.js with requireJS, since `module.exports` / require
+        // only work if things are in separate files. In most situations
+        // `module` is already undefined, but in VSCode is is not (gh-46).
+        // Explicitly clearing it makes ganja.js fall back to RequireJS.
+        let module = undefined;
+        """
+        js += read_ganja()
         js += """
         // take a closure on element before the next cell replaces it
         (function(element) {
@@ -85,7 +100,7 @@ def generate_notebook_js(scene, sig=None, grid=True, scale=1.0, gl=True):
                     // convert arrays of floats back to CGA elements.
                     data = data.map(x=>x.length==opts.mv_length?new Element(x):x);
                     // add the graph to the page.
-                    canvas = this.graph(data, {gl: opts.gl, conformal: opts.conformal, grid: opts.grid, scale: opts.scale, useUnnaturalLineDisplayForPointPairs: true});
+                    canvas = this.graph(data, opts.graph);
                     canvas.options.h = h;
                     canvas.options.p = p;
                     // make it big.
@@ -112,7 +127,6 @@ def generate_notebook_js(scene, sig=None, grid=True, scale=1.0, gl=True):
                 a.onclick = screenshot
                 var butnelem = element.append(a);
             });
-
         })(element);
         """
     else:
@@ -120,8 +134,9 @@ def generate_notebook_js(scene, sig=None, grid=True, scale=1.0, gl=True):
     return Javascript(js)
 
 
-def generate_full_html(scene, sig=None, grid=True, scale=1.0, gl=True):
-    script_json = _to_scene_string(scene)
+def generate_full_html(scene, sig=None, grid=True, scale=1.0, gl=True,
+                       default_color=Color.DEFAULT, default_static=False):
+    script_json = _to_scene_string(scene, default_color=default_color, default_static=default_static)
     if sig is not None:
         p = (sig > 0).sum()
         q = (sig < 0).sum()
@@ -174,12 +189,14 @@ def generate_full_html(scene, sig=None, grid=True, scale=1.0, gl=True):
         raise ValueError('Algebra not yet supported')
 
 
-def render_browser_script(scene, sig=None, grid=True, scale=1.0, gl=True, filename=None):
+def render_browser_script(scene, sig=None, *, filename=None,
+                          default_color=Color.DEFAULT, default_static=False, **kwargs):
     """
     If we have no jupyter and no cefpython we will be forced to generate html
     and render that in the users browser
     """
-    html_code = generate_full_html(scene, sig=sig, grid=grid, scale=scale, gl=gl)
+    html_code = generate_full_html(
+        scene, sig=sig, default_color=default_color, default_static=default_static, **kwargs)
     if filename is None:
         hash_object = hashlib.md5(html_code.encode())
         filename = hash_object.hexdigest() + '.html'
@@ -188,18 +205,18 @@ def render_browser_script(scene, sig=None, grid=True, scale=1.0, gl=True, filena
     webbrowser.open(filename)
 
 
-def render_notebook_script(scene, sig=None, grid=True, scale=1.0, gl=True):
+def render_notebook_script(scene, sig=None, **kwargs):
     """
     In a notebook we dont need to start cefpython as we
     are already in the browser!
     """
-    js = generate_notebook_js(scene, sig=sig, grid=grid, scale=scale, gl=gl)
+    js = generate_notebook_js(scene, sig=sig, **kwargs)
     display(js)
 
 
-def render_cef_script(scene="", sig=None, grid=True, scale=1.0, gl=True):
+def render_cef_script(scene="", sig=None, **kwargs):
     def render_script():
-        final_url = html_to_data_uri(generate_full_html(scene, sig=sig, grid=grid, scale=scale, gl=gl))
+        final_url = html_to_data_uri(generate_full_html(scene, sig=sig, **kwargs))
         run_cef_gui(final_url, "pyganja")
     p = Process(target=render_script)
     p.start()
@@ -221,41 +238,21 @@ def isnotebook():
         return False      # Probably standard Python interpreter
 
 
-def draw(objects, color=int('AA000000', 16), sig=None, grid=True, scale=1.0, browser_window=False, new_window=False, static=False, gl=True):
-    if JUPYTERAVAILABLE:
-        if isnotebook():
-            if not new_window:
-                render_notebook_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-            else:
-                if CEFAVAILABLE:
-                    if browser_window:
-                        render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-                    else:
-                        render_cef_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-                else:
-                    render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-        else:
-            if CEFAVAILABLE:
-                if browser_window:
-                    render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-                else:
-                    render_cef_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-            else:
-                render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
+def draw(objects, color=Color.DEFAULT, sig=None, *,
+         browser_window=False, new_window=False, static=False, **kwargs):
+    kwargs = dict(sig=sig, default_color=color, default_static=static, **kwargs)
+    if JUPYTERAVAILABLE and isnotebook() and not new_window:
+        render_notebook_script(objects, **kwargs)
+    elif CEFAVAILABLE and not browser_window:
+        render_cef_script(objects, **kwargs)
     else:
-        if CEFAVAILABLE:
-            if browser_window:
-                render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-            else:
-                render_cef_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
-        else:
-            render_browser_script(objects, sig=sig, grid=grid, scale=scale, gl=gl)
+        render_browser_script(objects, **kwargs)
 
 
-def _to_scene_string(objects):
+def _to_scene_string(objects, default_color=Color.DEFAULT, default_static=False):
     if isinstance(objects, list):
         sc = GanjaScene()
-        sc.add_objects(objects, color=color, static=static)
+        sc.add_objects(objects, color=default_color, static=default_static)
         return str(sc)
     elif isinstance(objects, str):
         return objects
@@ -265,7 +262,7 @@ def _to_scene_string(objects):
         sc = GanjaScene()
         try:
             print('Treating as iterable')
-            sc.add_objects([i for i in objects], color=color, static=static)
+            sc.add_objects([i for i in objects], color=default_color, static=default_static)
             return str(sc)
         except Exception:
             raise ValueError('The input cannot be interpreted, it is not a list of objects or ganja scene')
